// Fragment Shader  file "dynamic_shader.frag"
// static means no modifications to the mesh

#version 330 core

const struct light_attributes
{
  float conAtt;
  float linAtt;
  float quaAtt;
  float spotCosCutoff;
  float spotExponent;
  vec4 color;
} spotLight = light_attributes(0.025, 0.000025, 0.0000030, 0.75, 20.0, vec4(1.0,0.4,0.4,1.0));


uniform vec3 lightPosition;
uniform sampler2D colorTexture;
uniform sampler2DShadow shadow_map; // for SM: the shadow map texture

uniform vec4 mAmbient;
uniform vec4 mDiffuse;
uniform vec4 mSpecular;
uniform float shininess;
uniform float shininess_strength;
uniform vec3 spotDirection;
uniform int do_red;

in  vec2 ex_textCoords;
in  vec3 ex_normal;
in  vec3 ex_ecPosition;
in  vec3 ex_eyeVector;
in vec4 SM_tex_coord; // for SM: passed on from VS (the coordinates for Texture access)

out vec4 frag_Color;

float doShadowComparison()
{
	// check if point is in behind of light's view frustum
	if(SM_tex_coord.w < 0.0) return 1.0;
	return textureProj(shadow_map, SM_tex_coord);
}


void calcSpotlight()
{
  float nDotVP;			// normal . light direction
  float nDotHV; 		// normal . light half vector
  float pf;			// power factor
  float spotDot; 		// cosine of angle between spotlight
  float spotAttenuation;	// spotlight attenuation factor
  float attenuation;		// computed attenuation factor
  float d;			// distance from surface to light source
  vec3 VP;			// direction from surface to light position
  vec3 halfVector;		// direction of maximum hightlights
  vec4 ambient;
  vec4 diffuse;
  vec4 specular;

  // Compute vector from surface to light position
  VP = vec3(lightPosition - ex_ecPosition);

  // Compute distance between surfface and light position
  d = length(VP) / 3;

  // Normalize the vector from surface to light position
  VP = normalize(VP);

  // Compute attenuation
  attenuation = 1.0  /	(spotLight.conAtt +
			spotLight.linAtt * d +
			spotLight.quaAtt * d * d);

  // See if point on surface is inside cone of illumination
  spotDot = dot(-VP, normalize(spotDirection));
  if (spotDot < spotLight.spotCosCutoff)
    spotAttenuation = 0.0;
  else
    spotAttenuation = pow(spotDot, spotLight.spotExponent);

  // Combine the spotlight and distance attenuation
  attenuation *= spotAttenuation;

  halfVector = normalize(VP + ex_eyeVector);

  nDotVP = max(0.0, dot(normalize(ex_normal), VP));
  nDotHV = max(0.0, dot(normalize(ex_normal), halfVector));

  if (nDotVP == 0.0)
    pf = 0.0;
  else
    pf = pow(nDotHV, shininess);

  nDotVP = max(nDotVP, 0.01);
  
  specular = mSpecular * pf * attenuation * shininess_strength;
  attenuation = max(0.015, attenuation);
  ambient = mAmbient * attenuation;
  diffuse = mDiffuse * nDotVP * attenuation;

  frag_Color = clamp(ambient + diffuse, 0.0, 1.0) * vec4(texture2D(colorTexture, ex_textCoords.st).rgb , 1);
  frag_Color.rgb += specular.rgb;

  //for SM: only show shadow if in spotlight
  if(spotAttenuation != 0.0)
  {
	float inShadow = doShadowComparison();
	// darken fragment color if in shadow...
	if (inShadow == 0.0 || nDotVP < 0.0)
	{
		//frag_Color.rgb = frag_Color.rgb *0.04;
		ambient = mAmbient * 0.1;
		diffuse = mDiffuse * nDotVP * 0.1;

		frag_Color = clamp(ambient + diffuse, 0.0, 1.0) * vec4(texture2D(colorTexture, ex_textCoords.st).rgb , 1);
	}
  }
  if(do_red == 1)
  {
	  frag_Color.r = 1.0;
  }

}

void main(void)
{
//  frag_Color = vec4(texture2D(colorTexture, ex_textCoords.st).rgb , 1);
  calcSpotlight();
}
